<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Zack Bartel   <bartel@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/**
  * DAOWriter.php - contains the {@link DAOWriter} class
  * used to write the DAOs to the filesystem, used during installation
  * or any new modules/upgrades that involve a change to the database
  * structure.
  *
  * @author $Author$
  * @filesource
  * @package    install
  * @subpackage classes
  */

require_once('PEAR.php');

// a list of constants for this class

/** Nothing has been attempted yet: no status to report */
define("OK",1);
/** There are insufficient or no options set for DB_DataObject */
define("MISSING_PROPS", 2);
/** The schema_location directory is not writable */
define("NOT_WRITABLE", 4);
/** Missing the <dbname>.links.ini file */
define("MISSING_LINKS_INI", 8);
/** Unknown Error */
define("UNKNOWN", 16);

/** The INI file containing status and remedy messages for DAOWriter */
// TODO internationalization of DAOWriter messages
define("DAOWriterINI", CLASSDIR."/DAOWriter.ini");

/**
  * The DAOWriter class
  *
  * @author $Author: $
  * @package    install
  * @subpackage classes
  */
class DAOWriter
{

// {{{ vars

    /** holds one of the constants describing
      * the state of this object */
    var $status; 

    /** the data (verbatim) to be written to an ini file
      * if needed
      */
    var $ini_data;

    /** DB_DataObject options */
    var $options;


// }}}

// {{{ constructor()

    /**
      * Constructor
      */
    function DAOWriter()
    {
        // we're a naked object thus far
        $this->status = OK;

        // get the DB_DataObject properties
        $this->getOptions();

        // continue only with valid options
        if (!is_array($this->options) || empty($this->options)) {
            $this->addStatus(MISSING_PROPS);
        } else {

            // test that the schema_location directory is writable by this user
            // this does not return a 'false' status, as there are still options
            //$writable = is_writable($this->options['schema_location']);
            //if (!$writable) {
              //  $this->addStatus(NOT_WRITABLE);
            //}
        } // valid options
    } // DAOWriter

// }}}
// {{{ addStatus()
    
    /**
      * Adds a status code to this object.
      * Duplicate status codes are ignored.
      *
      * @param  int status code from the defined constants
      */
    function addStatus($pStatus)
    {
        // if we're setting the status to OK, clear the other status bits
        if ($pStatus == OK) {
            $this->status &= OK;
        } else {
            // clear the 'OK' bit
            $this->status &= ~ OK;

            // only add the value if its bit is not already set
            $this->status |= $pStatus;
        }
    }

// }}}
// {{{ getStatus()

    /**
      * Returns the status
      *
      * @return int $this->status
      */
    function getStatus()
    {
        return $this->status;
    }

// }}}
// {{{ getMessages()
    /**
      * Describes, in writing, the status code(s)
      *
      * @return array   status and remedy for each error status
      *                 array(array('status'=>blah, 'remedy'=>blah),...)
      */
    function getMessages()
    {
        global $conf;

        // read in the ini file
        $ini = parse_ini_file(DAOWriterINI, true);

        // re-index the array to match the status codes by integer instead of by string
        foreach ($ini as $section => $contents) {
            foreach ($contents as $key => $val) {
                $ini[$section][constant($key)] = $val;
                unset($ini[$section][$key]);
            }
        }

        // add each relevant message to the results array
        $results = array();
        for ($code = OK; $code <= $this->status; $code <<= 1) {
            if (!($code & $this->status)) { continue; }

            $error['status'] = $this->postProcessIni($ini['status'][$code]);
            $error['remedy'] = $this->postProcessIni($ini['remedy'][$code]);

            $results[] = $error;
        }

        return $results;
    }

// }}}
// {{{ getOptions()

    /**
      * Returns a copy of the DB_DataObject PEAR properties
      *
      * @access private
      * @return array   copy of the DB_DataObject PEAR properties
      */
    function getOptions()
    {
        global $conf;

        // if there's a global $conf, there's an already-configured DB_DataObject
        if (!$conf) {
            setup();
        }

        // get the DB_DataObject properties
        $opts = PEAR::getStaticProperty('DB_DataObject','options');

        // if the debug level has been set, do not propogate it
        unset ($opts['debug']);

        // copy them so we're not messing up the real properties
        $_options = array();
        foreach ($opts as $key=>$val) {
            $_options[$key] = $val;
        }

        $this->options = $_options;
    }

// }}}
// {{{ verifyLinksIni()

    /**
      * Ensure there is a <dbname>.links.ini file.
      * Sets status to MISSING_LINKS_INI if INI filename doesn't match
      * or cannot be created from a .dist version.
      *
      * @return boolean true if links.ini file exists and name
      *                 matches database name in $options
      */
    function verifyLinksIni($write_file = true)
    {
        global $conf;

        // test is only valid if we're error-free so far
        /*
        if ($this->status != OK) {
            return false;
        }
        */

        /* the <dbname>.links.ini needs to be named appropriately */

        $dir        = $this->options['schema_location'];
        $db         = $conf['database']['database'];
        $inifile    = "$db.links.ini";

        // get the list of files
        $handle = opendir($dir);

        $files = array();
        while ($file = readdir($handle)) {
            $files[] = $file;
        }

        if (in_array($inifile, $files)) {
            // if <dbname>.links.ini already exists, we're done
            return true;
        } else {
            $result = false;
            if (in_array("maintain.links.ini.dist", $files)) {
                // if it doesn't, we need to create it from the .dist file
                if ($write_file)    {
                    $result = copy("$dir/maintain.links.ini.dist", "$dir/$db.links.ini");
                }
                else    {
                    $this->addStatus(MISSING_LINKS_INI);
                    // let the makefile do a "cp $dir/maintain.links.ini.dist $dir/$db.links.ini"
                    return "make db";
                }
            }
            return $result;
        }
    }

// }}}
// {{{ createTables()

    /**
      * Executes createTables.php for DB_DataObject using the proper options.
      * Write the necessary .ini information to 
      *
      * @param  boolean $verify_links_ini   check that /class/dao/<dbname>.links.ini 
      *                                     matches the name of the database 
      *                                     given in maintain.cfg
      * @param  integer $verbosity  0-5 range of DB_DataObject::debugLevel output
      *                             0 = no output, 5 = max debugging info
      */
    function createTables($verify_links_ini = true, $verbosity = 0)
    {
        // only try this if everything's A-OK
        if ($this->status != OK) {
            return false;
        }

        // set the INI data for createTables.php
        $this->ini_data = "[DB_DataObject]\n\n";
        foreach($this->options as $name=>$value) {
            $this->ini_data .= "$name\t= $value\n";
        }

        require_once("DB/DataObject/Generator.php");

        DB_DataObject::debugLevel($verbosity);

        $generator = new DB_DataObject_Generator();
        $generator->start();

        // there is no way to test if this was successful!!
       
        DB_DataObject::debugLevel();
    }

// }}}
// {{{ postProcessIni()

    /**
      * Replaces EVAL_CONST(.*) with the value of a known constant within
      * a .ini file key=>value pair (just the value part).
      *
      * @static
      * @access private
      * @param  string  $val    the value of an .ini key-value pair to be parsed
      *                         for EVAL_CONST(CONSTANT)
      * @return string          the result of the replacement, 
      *                         i.e. EVAL_CONST(BOO) --> constant("BOO")
      */
    function postProcessIni($val)
    {
        $constants = array();
        ereg("EVAL_CONST\(.*\)", $val, $constants);

        $result = $val;
        foreach ($constants as $eval_const) {
            $const = ereg_replace("EVAL_CONST\((.*)\)", "\\1", $eval_const);
            $result = str_replace("$eval_const", constant($const), $result);
        }

        return $result;
    }

// }}}

// Formatting Functions

// {{{ getFormattedMessages()

    /**
      * Converts the getMessages() array into formatted output, like
      * an HTML datalist or plain text with spaces/tabs/newlines.
      *
      * @param  string  format: either 'html' or 'txt'
      * @param  array   set of errors directly from DAOWriter::getMessages().
      *                 if not provided, function will call getMessages first.
      *
      * @return mixed   HTML object or plain string
      */
    function getFormattedMessages($format = 'txt', $messages = false)
    {
        $result = false;
        if (!$messages)
            $messages = $this->getMessages();

        switch ($format) {

        case 'txt':
            $result = $this->statusInTxt($messages);
            break;

        case 'html':
        default:
            $result = $this->statusInHtml($messages);
        }

        return $result;
    }

// }}}
    
// {{{ statusInHTML()

    /**
      * Converts the getMessages() array into an HTML datalist.  This
      * implicitly requires that the phphtmllib libs already be loaded.
      *
      * @access private
      * @param  array   set of errors directly from DAOWriter::getMessages().
      *                 if not provided, function will call getMessages first.
      * @return object  datalist (DLTag object) containing errors and remedies
      */
    function statusInHTML($messages = false)
    {
        $html = container();

        if (!$messages)
            $messages = $this->getMessages();

        foreach ($messages as $error) {
            $html->add(html_dt(html_b(_("Error:"))), html_dd($error['status']));
            $html->add(html_dt(html_b(_("Remedy:"))), html_dd($error['remedy']));
        }
      
        return html_dl($html);
    }

// }}}
// {{{ statusInTxt()

    /**
      * Converts the getMessages() array into a formatted, plain-text representation
      * with newlines, etc.  Specifically designed with the command-line in mind.
      *
      * @access private
      * @param  array   set of errors directly from DAOWriter::getMessages().
      *                 if not provided, function will call getMessages first.
      * @return string  plain string representation of errors, formatted with tabs/spaces/newlines
      */
    function statusInTxt($messages = false)
    {
        $txt = '';

        foreach ($messages as $error) {
            $txt .= "Error:\n";
            $txt .= "\t{$error['status']}\n";
            $txt .= "Remedy:\n";
            $txt .= "\t{$error['remedy']}\n";
        }

        return $txt;
    }

// }}}

}

?>
